package org.apidesign.apifest08.currency;

import java.util.Currency;


/**
 * Creates default {@link Convertor} implementations.
 * @author lukas
 *
 */
public class ConvertorFactory {
	private static final DefaultConvertorFactory DEFAULT_FACTORY = new DefaultConvertorFactory();
	
	private ConvertorFactory()
	{
		//nothing
	}
		
	/**
	 * Creates {@link Convertor} that converts from sourceCurrency to destinationCurrency with stored exchange rate.
	 * @param sourceCurrency
	 * @param destinationCurrency
	 * @return
	 * @throws UnsupportedConversionException when exchange rate between currencies is not known.
	 */
	/*
	 * Only one of the createConveror methods is needed. The assignment is not explicit where the exchange rate should be set.
	 */
	public static final Convertor createConvertor(Currency sourceCurrency, Currency destinationCurrency) throws UnsupportedConversionException
	{
		return DEFAULT_FACTORY.getConvertor(sourceCurrency, destinationCurrency);
	}
	/**
	 * Creates {@link Convertor} that converts from sourceEquivalent.currency to destinationEquivalent.currency. 
	 * Exchange rate is set as equivalents. It means if you want to create USD to CZK convertor where USD1 = CZK17 
     * call createConvertor(new MoneyImpl(1, USD), new MoneyImpl(17, CZK)).
	 * @param sourceEquivalent
	 * @param destinationEquivalent
	 * @return
	 */
	/*
	 * Only one of the createConveror methods is needed. The assignment is not explicit where the exchange rate should be set.
	 */
	public static final Convertor createConvertor(Money sourceEquivalent, Money destinationEquivalent)
	{
		return new DefaultConvertor(sourceEquivalent.getAmount(), destinationEquivalent.getAmount(), sourceEquivalent.getCurrency(), destinationEquivalent.getCurrency());
	}
	

}
