package org.apidesign.apifest08.currency;


import java.math.BigDecimal;
import java.util.Currency;
import java.util.HashMap;
import java.util.Map;

/**
 * Default factory for convertors. Basically it just keeps exchange rates for given currency combinations.
 * @author lukas
 *
 */
class DefaultConvertorFactory {
	private static final Currency SKK = Currency.getInstance("SKK");
	private static final Currency USD = Currency.getInstance("USD");
	private static final Currency CZK = Currency.getInstance("CZK");
	private Map<String, Convertor> convertorMap = new HashMap<String, Convertor>();
	
	public 	DefaultConvertorFactory()
	{
		addConvertor(CZK,USD,BigDecimal.valueOf(17),BigDecimal.valueOf(1));
		addConvertor(CZK,SKK,BigDecimal.valueOf(80),BigDecimal.valueOf(100));
	}

	private void addConvertor(Currency sourceCurrency, Currency destinationCurrency, BigDecimal sourceEquivalent, BigDecimal destinationEquivalent) {
		DefaultConvertor convertor = new DefaultConvertor(sourceEquivalent, destinationEquivalent, sourceCurrency, destinationCurrency);
		convertorMap.put(getConvertorKey(sourceCurrency, destinationCurrency), convertor);
		convertorMap.put(getConvertorKey(destinationCurrency, sourceCurrency), convertor.revert());
	}
		
	public Convertor getConvertor(Currency sourceCurrency, Currency destinationCurrency) throws UnsupportedConversionException
	{
		String convertorKey = getConvertorKey(sourceCurrency, destinationCurrency);
		Convertor result = convertorMap.get(convertorKey);
		if (result!=null)
		{
			return result;
		}
		else
		{
			throw new UnsupportedConversionException("Conversion from "+sourceCurrency+" to "+destinationCurrency+" is not supported");
		}
	}

	private String getConvertorKey(Currency sourceCurrency,	Currency destinationCurrency) {
		return sourceCurrency.getCurrencyCode()+destinationCurrency.getCurrencyCode();
	}
}
