package org.apidesign.apifest08.test;

import java.math.BigDecimal;
import java.util.Currency;
import junit.framework.TestCase;
import org.apidesign.apifest08.currency.Convertor;
import org.apidesign.apifest08.currency.ConvertorFactory;

/** Finish the Convertor API, and then write bodies of methods inside
 * of this class to match the given tasks. To fullfil your task, use the
 * API define in the <code>org.apidesign.apifest08.currency</code> package.
 * Do not you reflection, or other hacks as your code
 * shall run without any runtime permissions.
 */
public class Task1Test extends TestCase {
    public Task1Test(String testName) {
        super(testName);
    }

    @Override
    protected void setUp() throws Exception {
    }

    @Override
    protected void tearDown() throws Exception {
    }

    /** Create convertor that understands two currencies, CZK and
     *  USD. Make 1 USD == 17 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     *
     * @return prepared convertor ready for converting USD to CZK and CZK to USD
     */
    public static Convertor createCZKtoUSD()
    {
        return (ConvertorFactory.getConvertor("CZK", "USD"));
    }

    /** Create convertor that understands two currencies, CZK and
     *  SKK. Make 100 SKK == 80 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     * 
     * @return prepared convertor ready for converting SKK to CZK and CZK to SKK
     */
    public static Convertor createSKKtoCZK()
    {
        return (ConvertorFactory.getConvertor(Currency.getInstance("SKK"), Currency.getInstance("CZK")));
    }
    
    public static Convertor createUSDtoUSD()
    {
        return (ConvertorFactory.getConvertor(Currency.getInstance("USD"), Currency.getInstance("USD")));
    }
    
    /** Use the convertor from <code>createCZKtoUSD</code> method and do few conversions
     * with it.
     */
    public void testCurrencyCZKUSD() throws Exception {
        Convertor  c = createCZKtoUSD();
        BigDecimal result;
        
        // convert $5 to CZK using c:
        // assertEquals("Result is 85 CZK");
        result = c.convertFrom(BigDecimal.valueOf(5));
        assertEquals(new BigDecimal("85.00"), result);

        // convert $8 to CZK
        // assertEquals("Result is 136 CZK");
        result = c.convertFrom(BigDecimal.valueOf(8));
        assertEquals(new BigDecimal("136.00"), result);

        // convert 1003CZK to USD
        // assertEquals("Result is 59 USD");
        result = c.convertTo(BigDecimal.valueOf(1003));
        assertEquals(new BigDecimal("59.00"), result);
    }

    /** Use the convertor from <code>createSKKtoCZK</code> method and do few conversions
     * with it.
     */
    public void testCurrencySKKCZK() throws Exception {
        Convertor c = createSKKtoCZK();
        BigDecimal result;
        
        // convert 16CZK using c:
        // assertEquals("Result is 20 SKK");
        result = c.convertFrom(BigDecimal.valueOf(16));
        assertEquals(new BigDecimal("20.00"), result);
                        
        // convert 500SKK to CZK
        // assertEquals("Result is 400 CZK");
        result = c.convertTo(BigDecimal.valueOf(500));
        assertEquals(new BigDecimal("400.00"), result);
   }

    /** Use the convertor from <code>createSKKtoCZK</code> method and do few conversions
     * with it.
     */
    public void testCurrencyUSDUSD() throws Exception {
        Convertor c = createUSDtoUSD();
        BigDecimal result;
        
        // convert 1USD using c:
        // assertEquals("Result is 1 USD");
        result = c.convertFrom(BigDecimal.valueOf(1));
        assertEquals(new BigDecimal("1.00"), result);
                        
        // convert 500USD to USD
        // assertEquals("Result is 500 USD");
        result = c.convertTo(BigDecimal.valueOf(500));
        assertEquals(new BigDecimal("500.00"), result);
   }
}

